@@ -3,8 +3,8 @@
  * Copyright (c) Paul Kinnucan 2000. All Rights Reserved.
  * Copyright (c) Charles Hart 2000. All Rights Reserved.
  *
- * $Revision: 1.2 $ 
- * $Date: 2001/08/16 03:29:12 $ 
+ * $Revision: 1.4 $ 
+ * $Date: 2002/09/14 03:20:48 $ 
  *
  * DelegateFactory is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,7 +37,7 @@ import java.util.Enumeration;
  * multiple interfaces when invoked from the command line.
  *
  * @author Charles Hart, Eric D. Friedman, and Paul Kinnucan
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.4 $
  */
 
 public class DelegateFactory extends MethodFactory
@@ -49,6 +49,8 @@ public class DelegateFactory extends MethodFactory
 
   /** The interface factory. */
   static DelegateFactory delegateFactory;
+
+  private String delegee;
   
   public DelegateFactory() {}
 
@@ -142,44 +144,41 @@ public class DelegateFactory extends MethodFactory
 
 
   /**
-   * Makes an implementation of an interface. This method delegates the creation
-   * of the implementation to makeInterfaceInternal.
+   * Makes an expression for generating the implementation of an
+   * interface. This method delegates the creation of the
+   * implementation to makeInterfaceInternal.
    *
    * @param name Name of interface to be implemented.
-   * @param javadoc If <code>true</code> generate skeletal Javadoc for the implementation.
-   * @param truncate If <code>true</code>, truncate package specifier when generating code.
-   * @param newline If <code>true</code>, insert a newline after opening brace.
-   * 
+   * @param truncate If <code>true</code>, truncate package specifier
+   * when generating code.
    */
-  public static void makeDelegates(String attributeName,
-                                   String className, 
-                                   boolean javadoc,
-                                   boolean truncate,
-                                   boolean newline) {
+  public static void makeDelegatesExpression(String attributeName,
+					     String className, 
+					     boolean truncate) {
 
     if (delegateFactory == null)
       delegateFactory = new DelegateFactory();
 
     delegateFactory.flush();
-    delegateFactory.makeInterfaceInternal(attributeName, className, javadoc, 
-                                          truncate, newline);
-
+    delegateFactory.makeInterfaceExpressionInternal 
+	(attributeName, className, truncate);
   }
 
- /**
-   * Makes an implementation of an interface.
+  /**
+   * Makes an expression for generating the implementation of an interface.
    *
    * @param name Name of interface to be implemented.
-   * @param javadoc If <code>true</code> generate skeletal Javadoc for the implementation.
-   * @param truncate If <code>true</code>, truncate package specifier when generating code.
-   * @param newline If <code>true</code>, insert a newline after opening brace.
+   * @param javadoc If <code>true</code> generate skeletal Javadoc for
+   * the implementation.
+   * @param truncate If <code>true</code>, truncate package specifier
+   * when generating code.
+   * @param newline If <code>true</code>, insert a newline after
+   * opening brace.
    * 
    */
-  private void makeInterfaceInternal(String attributeName,
-                                     String className,
-                                     boolean javadoc,
-                                     boolean truncate,
-                                     boolean newline) {
+  private void makeInterfaceExpressionInternal
+      (String attributeName, String className, boolean truncate) {
+    delegee = attributeName;
     try {
       process(className, truncate);
     }
@@ -193,91 +192,86 @@ public class DelegateFactory extends MethodFactory
       return;
     }
 
-    dump(new PrintWriter( System.out, true),
-         attributeName,
-         javadoc,
-         truncate,
-         newline );
+    dumpExpression(new PrintWriter(System.out, true), truncate);
   }
 
 
   public static void getImportedClasses() {
-    String res = "(list ";
-    Enumeration i = delegateFactory.imports.keys();
-    while (i.hasMoreElements()) {
-      Class c = (Class) i.nextElement();
-      res += "\"" + c.getName() + "\" ";
-    }
-    res += ")";
-    println(res);
+    println(delegateFactory.getImportsAsList());
   }
 
-
-  public String getMethodSkeleton(Signature sig,
-                                  boolean javadoc,
-                                  boolean newline,
-                                  String attributeName)
-  {
-    String res = "";
-    res += "\n" ;
-    if (javadoc)
-      res += sig.toJavaDoc() + "\n";
-    if (newline)
-      {
-	res += sig  + "\n";
-	res += "{\n";
-      }
-    else
-      res += sig + " {\n";
-          
+  /**
+   * Return a default body for the implementation of the method described
+   * by <code>sig</code>.
+   *
+   * @param sig a <code>Signature</code> value
+   * @return a <code>String</code> value
+   */
+  protected String getDefaultBody (Signature sig) {
     Method m = sig.getMethod();
     Class cl = m.getReturnType();
+    String defBody = "";
 
-    if (! cl.getName().equals("void"))
-      res += "  return "+attributeName+"."+m.getName()+
-             "("+sig.getParameterNames(m.getParameterTypes())+");\n";
-    else
-      res += "  "+attributeName+"."+m.getName()+
-             "("+sig.getParameterNames(m.getParameterTypes())+");\n";
-
-    res += "}\n";
-    return res;
+    defBody = delegee + "." + m.getName() 
+	+ "(" + sig.getParameterNames() + ");";
+    if (! cl.getName().equals("void")) {
+      defBody = "return " + defBody;
+    }
+    return defBody;
   }
 
-  public void dump(PrintWriter out,
-                   String attributeName,
-                   boolean javadoc,
-                   boolean truncate,
-                   boolean newline)
+  public void dumpExpression(PrintWriter out, boolean truncate)
   {
-    StringBuffer buf = new StringBuffer("\"");
+    StringBuffer buf = new StringBuffer
+	("(jde-wiz-gen-delegation-methods (list ");
 
     Enumeration declaring_classes = interfaces.keys();
     while (declaring_classes.hasMoreElements())
     {
       String interf = (String)declaring_classes.nextElement();
       Vector v = (Vector)interfaces.get(interf);
-      buf.append("// Code for delegation of ");
-      buf.append(interf);
-      buf.append(" methods to ");
-      buf.append(attributeName);
-      buf.append("\n");
       Enumeration e = v.elements();
+      boolean isFirst = true;
       while (e.hasMoreElements())
       {
         Signature sig = (Signature)e.nextElement();
-        buf.append(getMethodSkeleton(sig, javadoc, newline, attributeName));
+	if (interf.equals ("java.lang.Object")
+	    && Modifier.isFinal(sig.getMethod().getModifiers())) {
+	    continue;
+	}
+	if (isFirst) {
+	    buf.append ("(quote ");
+	    buf.append("\"Code for delegation of ");
+	    buf.append(interf);
+	    buf.append(" methods to ");
+	    buf.append(delegee);
+	    buf.append("\")");
+	    isFirst = false;
+	}
+	buf.append ("(quote ");
+        buf.append(getMethodSkeletonExpression(sig));
+	buf.append (")");
       } // end of while (e.hasMoreElements())
     }
-    buf.append("\"");
+    buf.append("))");
     println(buf.toString());
   }
 } // DelegateFactory
 
 /*
  * $Log: DelegateFactory.java,v $
- * Revision 1.2  2001/08/16 03:29:12  andyp
- * Update to JDE 2.2.7.1
+ * Revision 1.4  2002/09/14 03:20:48  andyp
+ * sync to jde 2.2.9beta12
+ *
+ * Revision 1.5  2002/06/06 05:12:44  paulk
+ * DefaultNameFactory now generates meaningful method parameter names based
+ * on the parameter type or the method name. Thanks to Ole Arndt.
+ *
+ * Revision 1.4  2002/05/14 06:38:44  paulk
+ * Enhances code generation wizards for implementing interfaces, abstract
+ * classes, etc., to use customizable templates to generate skeleton methods
+ * instead of hard-wired skeletons. Thanks to "Dr. Michael Lipp" <lipp@danet.de>
+ * for proposing and implementing this improvement.
  *
  * Revision 1.3  2000/08/03 04:31:20  paulk
  * Add support for generating a see secton in the Javadoc comment for a method. Thanks to raffael.herzog@comartis.com
