@@ -1,5 +1,5 @@
 ;;; jde-bug.el -- JDEbug Interface
-;; $Revision: 1.5 $ $Date: 2002/09/14 03:20:53 $ 
+;; $Revision: 1.6 $ $Date: 2002/10/10 21:47:52 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -395,6 +395,11 @@ consuming and slows down stepping through the code."
 
 	(list
 	 "Stack"
+	 `["Enable"
+	   jde-bug-toggle-stack-info
+	   ,(if jde-xemacsp :active :enable) t
+	   :style radio 
+	   :selected jde-bug-stack-info]
 	 ["Up"                        
 	  jde-bug-up-stack 
 	  (and
@@ -2588,6 +2593,9 @@ of the variable `jde-bug-stack-info'"
 (provide 'jde-bug)
 
 ;; $Log: jde-bug.el,v $
+;; Revision 1.6  2002/10/10 21:47:52  andyp
+;; misc fixes
+;;
 ;; Revision 1.5  2002/09/14 03:20:53  andyp
 ;; sync to jde 2.2.9beta12
 ;;
