@@ -1,11 +1,11 @@
 ;;; jde-dbo.el -- JDEbug output functions
-;; $Revision: 1.3 $ $Date: 2001/08/16 03:29:15 $ 
+;; $Revision: 1.5 $ $Date: 2002/09/14 03:20:55 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 1999, 2001 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 1999, 2001, 2002 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -54,62 +54,6 @@
 		  :state  (nth 3 thread-spec)
 		  :status (nth 4 thread-spec)))
 
-(defun jde-dbo-query-source-file (class)
-  (let ((source-file
-	 (read-file-name 
-	  (format "Cannot find %s source. Enter path: " class))))
-  (if (and
-       source-file
-       (file-exists-p source-file)
-       (not (file-directory-p source-file)))
-      (find-file-noselect source-file))))
-
-(defun jde-dbo-find-source-file (class file)
-  (interactive "sclass: \nsfile: ")
-  (let* ((source-dir
-	 (jde-db-search-src-dirs 
-	  file
-	  (jde-parse-get-package-from-name class)))
-	 (source-file
-	  (if source-dir
-	      (if (eq (aref source-dir (1- (length source-dir))) ?/)
-		  (concat source-dir file)
-		(concat source-dir "/" file))))
-	 (source-buffer
-	  (if (and
-	       source-file
-	       (file-exists-p source-file))
-	      (find-file-noselect source-file)
-	    (jde-dbo-query-source-file class))))
-    source-buffer))
-
-(defun jde-dbo-show-line (class file line)
-  "Shows the source at LINE in CLASS."
-  (let* ((buffer (jde-dbo-find-source-file class file))
-	 (window 
-	  (and buffer
-	       (or (get-buffer-window buffer)
-		   (selected-window))))
-	  pos) 
-    (if buffer
-	(progn
-	  (if (not (get-buffer-window buffer))
-	      (set-window-buffer window buffer))
-	  (save-excursion
-	    (set-buffer buffer)
-	    (jde-bug-install-jdebug-menu)
-	    (save-restriction
-	      (widen)
-	      (goto-line line)
-	      (setq pos (point))
-	      (setq overlay-arrow-string "=>")
-	      (or overlay-arrow-position
-		  (setq overlay-arrow-position (make-marker)))
-	      (set-marker overlay-arrow-position (point) (current-buffer)))
-	    (cond ((or (< pos (point-min)) (> pos (point-max)))
-		   (widen)
-		   (goto-char pos))))
-	  (set-window-point window overlay-arrow-position)))))	 
 
 (defun jde-dbo-command-result (id &rest args)
 "Returns the result of normally executing command specified by ID.
@@ -150,11 +94,14 @@ exception spec."
 	 (bpspec (if proc (jde-dbs-proc-get-bpspec proc spec-id)))
 	 (bp (if bpspec (oref bpspec breakpoint)))
 	 (file (if bp (oref bp file)))
-	 (line (if bp (oref bp line))))
+	 (line (if bp (jde-db-breakpoint-get-line bp))))
     (and proc file line
-	 (jde-dbs-proc-display-debug-message 
-	  proc
-	  (format "Resolved breakpoint set in %s at line %s." file line)))))
+	 (progn
+	   (oset bp status 'active)
+	   (jde-db-mark-breakpoint-active file line)
+	   (jde-dbs-proc-display-debug-message 
+	    proc
+	    (format "Resolved breakpoint set in %s at line %s." file line))))))
 
 (defun jde-dbo-error (proc-id message)
   (jde-dbs-display-debug-message proc-id message))
@@ -167,7 +114,7 @@ exception spec."
   (jde-dbs-proc-display-debug-message 
    (jde-dbs-get-target-process) exception))
 
-(defun jde-dbo-vm-start-event (process-id status process-state)
+(defun jde-dbo-vm-start-event (process-id process-status process-state)
   (let* ((process (jde-dbs-get-process process-id))
 	 (thread-id (nth 1 process-state))
 	 (thread-name (nth 2 process-state))
@@ -197,6 +144,25 @@ exception spec."
 ;;                                                                            ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(defvar jde-dbo-locals-open-nodes nil
+  "List of name of tree nodes opened.")
+
+(defun jde-dbo-locals-open-p (node-name)
+  "Return non-nil if NODE-NAME is the name of an open tree node."
+  (member node-name jde-dbo-locals-open-nodes))
+
+(defun jde-dbo-locals-update-open-nodes (tree)
+  "Update the list of open nodes `jde-dbo-locals-open-nodes'.
+Called after each folding/unfolding of the `tree-widget' TREE.
+See also the hook `tree-widget-after-toggle-fucntions'."
+  (let ((node-name (widget-get tree :node-name))
+        (open      (widget-get tree :open)))
+    (if open
+        (add-to-list 'jde-dbo-locals-open-nodes node-name)
+      (setq jde-dbo-locals-open-nodes  
+            (delete node-name jde-dbo-locals-open-nodes)))))
+  
+
 (defun jde-dbo-update-locals-buf (process thread frame)
   (let* ((cmd (jde-dbs-get-locals 
 	      "get locals" 
@@ -220,19 +186,30 @@ exception spec."
 	  (mapcar 'delete-overlay (car all))    
 	  (mapcar 'delete-overlay (cdr all)))) 
 
+      (make-local-hook 'tree-widget-after-toggle-functions)
+      (add-hook 'tree-widget-after-toggle-functions
+                'jde-dbo-locals-update-open-nodes nil t)
+
       (goto-char (point-min))
 
       ;; Insert the this object for this stack frame.
       (let* ((cmd (jde-dbs-get-this
 		   "get this"
 		   :process process
-		   :thread-id thread-id
+		   :thread-id thread
 		   :stack-frame-index frame))
 	     (this-obj (jde-dbs-cmd-exec cmd)))
 	(if (not (typep this-obj 'jde-dbs-java-null))
-	    (widget-create 'jde-widget-java-obj :tag "this" 
-			   :process process :object-id (oref this-obj :id))))	    
-
+            (progn
+              (let* ((id (oref this-obj :id))
+                     (open (concat "this" (number-to-string id))))
+              (widget-create 'jde-widget-java-obj
+                             :tag "this" 
+                             :node-name open
+                             :open (jde-dbo-locals-open-p open)
+                             :process process
+                             :object-id (oref this-obj :id))))))
+        
       ;; Insert the local variables for this stack frame.
       (while locals
 	(setq var (car locals) locals (cdr locals))
@@ -248,15 +225,31 @@ exception spec."
 				   :process process
 				   :object-id (oref var-value id)))
 		       (str-val (jde-dbs-cmd-exec cmd)))
-		  (widget-create 'tree-widget :tag var-tag :value t
+		  (widget-create 'tree-widget
+                                 :tag var-tag
+                                 :node-name var-tag
+                                 :open (jde-dbo-locals-open-p var-tag)
+                                 :value t
 				 (list 'tree-widget :tag str-val)))
-	      (widget-create 'jde-widget-java-obj :tag var-tag
-			     :process process :object-id (oref var-value :id))))
+	      (widget-create 'jde-widget-java-obj
+                             :tag var-tag
+                             :node-name var-tag
+                             :open (jde-dbo-locals-open-p var-tag)
+			     :process process
+                             :object-id (oref var-value :id))))
 	   ((typep var-value 'jde-dbs-java-array)
-	    (widget-create 'jde-widget-java-array :tag var-tag
-			   :process process :object var-value))
+	    (widget-create 'jde-widget-java-array
+                           :tag var-tag
+                           :node-name var-tag
+                           :open (jde-dbo-locals-open-p var-tag)
+			   :process process
+                           :object var-value))
 	   ((typep var-value 'jde-dbs-java-primitive)
-	    (widget-create 'tree-widget :tag var-tag :value t
+	    (widget-create 'tree-widget
+                           :tag var-tag
+                           :node-name var-tag
+                           :open (jde-dbo-locals-open-p var-tag)
+                           :value t
 			   (list 'tree-widget 
 				 :tag (format "%s" (oref var-value value)))))
 	   ((typep var-value 'jde-dbs-java-null)
@@ -277,6 +270,12 @@ exception spec."
     (oset process :stack stack)
     (oset process :stack-ptr 0)))
 
+(defvar jde-dbo-current-process nil "Used to keep track of the process
+used in the last breakpoint hit event, and watch point hit event.")
+
+(defvar jde-dbo-current-thread-id nil "Used to keep track of the thread id
+used in the last breakpoint hit event, and watch point hit event.")
+
 (defun jde-dbo-breakpoint-hit-event (process-id process-status process-state spec-id location a2 a3)
   (let ((process (jde-dbs-get-process process-id)))
     (if process
@@ -290,15 +289,20 @@ exception spec."
 	      (state-info (oref process state-info)))
 	  (if state-info
 	      (progn
-		(jde-dbs-proc-state-info-set state-info state reason thread-id thread-name)
-		(jde-dbo-update-locals-buf process thread-id 0)
-		(jde-dbo-update-stack process thread-id)
+		(jde-dbs-proc-state-info-set state-info state reason
+                                             thread-id thread-name)
+                (setq jde-dbo-current-process process)
+                (setq jde-dbo-current-thread-id thread-id)
+                (if jde-bug-local-variables
+                    (jde-dbo-update-locals-buf process
+                                               thread-id 0))
+                (if jde-bug-stack-info (jde-dbo-update-stack process thread-id))
 		(oset process steppablep t)
 		(jde-dbs-display-debug-message 
 		 process-id
 		 (format "Breakpoint hit at line %d in %s (%s) on thread %s. All threads suspended." 
 			 line-no class file thread-name))
-		(jde-dbo-show-line class file line-no)
+		(jde-db-set-debug-cursor class file line-no)
 		;; (message (selected-frame))
 		(when jde-bug-raise-frame-p (raise-frame))
 		)
@@ -319,15 +323,20 @@ exception spec."
 	      (state-info (oref process state-info)))
 	  (if state-info
 	      (progn
-		(jde-dbs-proc-state-info-set state-info state reason thread-id thread-name)
-		(jde-dbo-update-locals-buf process thread-id 0)
-		(jde-dbo-update-stack process thread-id)
+		(jde-dbs-proc-state-info-set state-info state reason
+                                             thread-id thread-name)
+                (setq jde-dbo-current-process process)
+                (setq jde-dbo-current-thread-id thread-id)
+                (if jde-bug-local-variables
+                    (jde-dbo-update-locals-buf process
+                                               thread-id 0))
+                (if jde-bug-stack-info (jde-dbo-update-stack process thread-id))
 		(oset process steppablep t)
 		(jde-dbs-display-debug-message 
-		 process-id
+		 proc-id
 		 (format "Stepped to line %d in %s (%s) on thread %s. All threads suspended." 
 			 line-no class file thread-name))
-		(jde-dbo-show-line class file line-no))
+		(jde-db-set-debug-cursor class file line-no))
 	    (message "Step event error: state info missing for process %d" proc-id)))
       (message "Step event error: could not find process %d." proc-id))))
 
@@ -353,6 +362,7 @@ exception spec."
     (when process
 	(jde-dbs-proc-display-debug-message process "vm disconnected...")
 	(setq overlay-arrow-position nil)
+	(jde-db-set-all-breakpoints-specified)
 	(jde-dbs-proc-set-state process "vm disconnected")
 	(when (jde-dbs-proc-set-contains-p jde-dbs-the-process-registry process)
 	  (jde-dbs-proc-move-to-morgue process)
@@ -498,10 +508,16 @@ exception spec."
 		      (jde-dbs-proc-state-info-set 
 		       state-info thread-state 
 		       thread-status thread-id thread-name)
-		      (jde-dbo-update-locals-buf process thread-id 0)
-		      (jde-dbo-update-stack process thread-id)
+                      (setq jde-dbo-current-process process)
+                      (setq jde-dbo-current-thread-id thread-id)
+                      (if jde-bug-local-variables
+                          (jde-dbo-update-locals-buf process
+                                                     thread-id 0))
+                      (if jde-bug-stack-info (jde-dbo-update-stack process
+                                                                   thread-id))
 		      (oset process steppablep t)
-		      (jde-dbo-show-line breakpoint-class breakpoint-file breakpoint-line)
+		      (jde-db-set-debug-cursor breakpoint-class breakpoint-file
+                                         breakpoint-line)
 		      (when jde-bug-raise-frame-p (raise-frame))
 
 		      (jde-dbs-display-debug-message 
@@ -527,4 +543,33 @@ The remaining elements are arguments to pass to the handler."
 
 
 
-(provide 'jde-dbo)
\ No newline at end of file
+(provide 'jde-dbo)
+;; $Log: jde-dbo.el,v $
+;; Revision 1.5  2002/09/14 03:20:55  andyp
+;; sync to jde 2.2.9beta12
+;;
+;; Revision 1.35  2002/02/04 05:47:17  paulk
+;; Added code to rehighlight breakpoints if the user kills a
+;; buffer for a source file that contains breakpoints and
+;; then reopens the file.
+;;
+;; Revision 1.34  2002/01/16 07:36:01  paulk
+;; Updated JDEbug to use most of the new generalized breakpoint functionality.
+;;
+;; Revision 1.33  2001/12/28 05:27:45  paulk
+;; Deleted jde-dbo-query-source-directory, jde-dbo-find-source-directory, and
+;; jde-dbo-show-line and replaced calls to these functions with calls to
+;; the equivalent functions in the jde-db package: jde-db-query-source-directory,
+;; jde-db-find-source-directory, and jde-db-set-debug-cursor.
+;;
+;; Revision 1.32  2001/12/04 06:14:29  paulk
+;; Replaced call to obsolete jde-bug-install-jdebug-menu with call to jde-bug-minor-mode.
+;;
+;; Revision 1.31  2001/11/27 21:06:49  jslopez
+;; Fixes some compilation messages.
+;; Adds code to be able to keep the state of
+;; the nodes in the JDEBug local variables tree.
+;;
+;; Revision 1.30  2001/11/23 15:58:47  jslopez
+;; Adds Log key word.
+;;
