@@ -1,5 +1,5 @@
 ;;; jde-jdb.el -- Debugger mode for jdb.
-;; $Revision: 1.1 $ $Date: 2002/09/14 03:21:02 $ 
+;; $Revision: 1.2 $ $Date: 2002/10/10 21:47:56 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -243,7 +243,8 @@ current stack location."
 		       (substring marker (match-beginning 2) (match-end 2)))))
 
 	  (jde-db-set-debug-cursor 
-	   (concat package "." class) (concat class ".java") line-no)))
+	   (if package (concat package "." class) class)
+	   (concat class ".java") line-no)))
     output))
 
 ;; Set Breakpoint command
@@ -1117,6 +1118,9 @@ file in the current buffer."
 
 ;; Change History
 ;; $Log: jde-jdb.el,v $
+;; Revision 1.2  2002/10/10 21:47:56  andyp
+;; misc fixes
+;;
 ;; Revision 1.1  2002/09/14 03:21:02  andyp
 ;; sync to jde 2.2.9beta12
 ;;
@@ -1191,4 +1195,4 @@ file in the current buffer."
 ;; Initial revision.
 ;;
 
-;;; end of jde-jdb.el
\ No newline at end of file
+;;; end of jde-jdb.el
