@@ -1,11 +1,11 @@
 ;; jde-run.el --- runs the Java app in the current buffer.
-;; $Revision: 1.4 $ $Date: 2001/08/16 03:29:21 $
+;; $Revision: 1.6 $ $Date: 2002/09/14 03:21:05 $
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: tools, processes
 
-;; Copyright (C) 1997, 1998, 1999, 2000, 2001 Paul Kinnucan
+;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 Paul Kinnucan
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 ;; Please send any comments, bugs, or upgrade requests to
 ;; Paul Kinnucan at paulk@mathworks.com.
 
+(require 'eieio)
 
 (defcustom jde-run-mode-hook nil
   "*List of hook functions run by `jde-run-mode' (see `run-hooks')."
@@ -58,18 +59,13 @@ directory specified by the path."
   :type 'file)
 
 
-(defcustom jde-run-java-vm "java"
-  "*Specify Java interpreter for non-Windows platforms."
-  :group 'jde-project
-  :type 'file)
-
-(defcustom jde-run-java-vm-w "javaw"
-  "*Specify Java interpreter for Windows platforms.
-On NTEmacs, the JDE must use the NTEmacs show window option in order
-to run the Java interpreter. This in turn requires that the JDE use
-the javaw version of the JDK Java interpreter to prevent a console
-window from appearing every time you run an application. Sound
-confusing? It is, but it works."
+(defcustom jde-vm-path ""
+  "*Path of the Java virtual machine executable. The path
+can include environment variables, e.g.,
+$JDK_HOME/bin/java. If the value of this variable is the
+empty string, the JDE uses the vm that comes with the
+version of the JDK specified by jde-jdk or, if jde-jdk
+is not set, the vm on the system command path."
   :group 'jde-project
   :type 'file)
 
@@ -214,6 +210,35 @@ Property Value field. You can specify as many properties as you like."
 	       (checkbox :format "%[%v%] %t \n"
 			 :tag "Classes loaded by a classloader.")))
 
+;; (makunbound 'jde-run-option-connect-to-debugger)
+(defcustom jde-run-option-connect-to-debugger (list t "Attach" nil)
+  "*If checked, this option allows the application to start and then
+connect to a debugger, for example, jdb or JDEbug. \"DebuggerConnect
+Mode\" specifies the method for establishing the connection. The
+options are \"Attach\" (allow a debugger to attach to the application
+after the applicaiton starts) or \"Client\" (start the application and
+then try to connect the application to a listening debugger). On Unix
+ systems, the \"Address\" option specifies the port number
+of the socket used to connect the debugger and the application. On
+Windows systems, it specifies an arbitrary identifier. If \"Address\"
+is nil, the JDEE uses a default value. The default is \"javadebug\" on
+Windows systems and 4444 on Unix systems."
+  :group 'jde-run-options
+  :type  '(list
+	   (checkbox :format "%[%v%] %t \n\n"
+                     :tag "Enabled")
+	   (radio-button-choice :format "  %t\n%v" 
+				:indent 2
+                                :tag "Debugger Connect Mode:"
+	    (const "Attach")
+	    (const "Listen"))
+	   (choice :format "\n  %[Address%] %v\n\n"
+	    (const :menu-tag "Use Default"
+                   nil)
+	    (string :menu-tag "Specify" :tag "Value"))))
+	  
+
+
 (defcustom jde-run-option-vm-args nil
   "*Specify arguments to be passed to the Java vm.
 This option allows you to specify one or more arguments to be passed
@@ -225,7 +250,6 @@ the JDE Run Option variable set."
   :group 'jde-run-options
   :type '(repeat (string :tag "Argument")))
 
-
 (defcustom jde-run-option-application-args nil
   "*Specify command-line arguments to pass to the application.
 The JDE passes the specified arguments to the application on
@@ -263,7 +287,7 @@ virtual machine that runs appletviewer."
 (defcustom jde-run-executable ""
   "*Specifies the executable to be run by the JDE's run command.
 If you do not specify an executable, the JDE runs the vm specified
-by `jde-run-java-vm' or `jde-run-java-vm-w'."
+by `jde-run-get-vm'."
   :group 'jde-project
   :type 'file)
 
@@ -300,22 +324,6 @@ whitespace is considered a parameter."
       (setq i (+ i 1)))
     str))
 
-;;;###autoload
-(defun jde-run-set-vm (vm)
-  "Specifies the Java interpreter used to run Java applications
-on non-Windows platforms . The default is java."
-  (interactive
-   "sEnter name of Java interpreter: ")
-  (setq jde-run-java-vm vm))
-
-;;;###autoload
-(defun jde-run-set-vm-w (vm)
-  "Specifies the Java interpreter used to run Java applications
-on Windows platforms . The default is javaw."
-  (interactive
-   "sEnter name of Java interpreter: ")
-  (setq jde-run-java-vm-w vm))
-
 ;;;###autoload
 (defun jde-run-set-app (app)
   "Specify the name of the application class to run."
@@ -362,69 +370,76 @@ panel to specifying the applet document."
       (setq jde-run-applet-doc nil)
     (setq jde-run-applet-doc doc)))
 
-(defun jde-run-get-vm-args ()
-  "Builds a command-line argument string to pass to the Java vm.
-This function builds the string from the values of the JDE
-Run Option panel variables."
-  (let (options
-	(memory-unit-abbrevs
-	 (list (cons "bytes" "")
-	       (cons "kilobytes" "k")
-	       (cons "megabytes" "m"))))
 
-    ;; Set classic-mode option
+(defclass jde-run-vm ()
+  ((version          :initarg :version
+		     :type string
+		     :initform ""
+		     :documentation
+		     "Compiler version.")
+   (path             :initarg :path
+		     :type string
+		     :documentation
+		     "Path of the compiler executable.")
+   (buffer           :initarg :buffer
+	             :type buffer
+	             :documentation
+	             "Compilation buffer")
+   (main-class        :initarg :main-class
+		     :type string
+		     :documentation
+		     "Name of main class."))
+    "Class of Java virtual machines.")
+
+(defmethod jde-run-classpath-arg ((this jde-run-vm))
+  "Returns the classpath argument for this vm."
+  (let ((classpath
+	 (if jde-run-option-classpath
+	     jde-run-option-classpath
+	   jde-global-classpath))
+	(symbol
+	 (if jde-run-option-classpath
+	     'jde-run-option-classpath
+	   'jde-global-classpath)))
+    (if classpath
+	(list
+	 "-classpath"
+	 (jde-build-classpath
+	  classpath symbol)))))
+
+(defmethod jde-run-classic-mode-arg ((this jde-run-vm))
+  "Get classic-mode option>"
     (if jde-run-classic-mode-vm
-	(setq options
-	      (nconc
-	       (list "-classic"))))
+	(list "-classic")))
 
-    ;; Set the classpath option. Use the local
-    ;; classpath, if set; otherwise, the global
-    ;; classpath.
-    (let ((classpath
-	   (if jde-run-option-classpath
-		 jde-run-option-classpath
-	       jde-global-classpath))
-	  (symbol
-	   (if jde-run-option-classpath
-		 'jde-run-option-classpath
-	       'jde-global-classpath)))
-      (if classpath
-	  (setq options
-		(nconc
-		 options
-		 (list
-		  "-classpath"
-		  (jde-build-classpath classpath symbol))))))
     
-
-    ;; Set the verbose options.
+(defmethod jde-run-verbose-arg ((this jde-run-vm))
+  "Set the verbose options."
     (let ((print-classes-loaded
 	   (nth 0 jde-run-option-verbose))
 	  (print-memory-freed
 	   (nth 1 jde-run-option-verbose))
 	  (print-jni-info
-   (nth 2 jde-run-option-verbose)))
-      (if print-classes-loaded
-	  (setq options (nconc options (list "-v"))))
-      (if print-memory-freed
-	  (setq options (nconc options '("-verbosegc"))))
-      (if print-jni-info
-	  (setq options (nconc options '("-verbosejni")))))
+	   (nth 2 jde-run-option-verbose)))
+      (append
+       (if print-classes-loaded (list "-v"))
+       (if print-memory-freed (list "-verbosegc"))
+       (if print-jni-info (list "-verbosejni")))))
 
-    ;; Set properties arguments.
-    (if jde-run-option-properties
-	(let ((count (length jde-run-option-properties))
-	      (n 0))
-	  (while (< n count)
-	    (let ((prop (nth n jde-run-option-properties)))
-	      (setq options 
-		    (nconc options
-			   (list (concat "-D" (car prop) "=" (cdr prop))))))    
-	    (setq n (1+ n)))))
+(defmethod jde-run-property-args ((this jde-run-vm))
+    "Get property arguments."
+    (mapcar
+     (lambda (prop)
+       (format "-D%s=%s" (car prop) (cdr prop)))
+     jde-run-option-properties))
 
-    ;; Set heap size options.
-    (let* ((start-cons (nth 0 jde-run-option-heap-size))
+(defmethod jde-run-heap-size-args ((this jde-run-vm))
+   "Get heap size arguments."
+    (let* ((memory-unit-abbrevs
+	    (list (cons "bytes" "")
+	       (cons "kilobytes" "k")
+	       (cons "megabytes" "m")))
+	   (start-cons (nth 0 jde-run-option-heap-size))
 	   (start-size (format "%d%s" (car start-cons) 
 			       (cdr (assoc (cdr start-cons)
 				      memory-unit-abbrevs))))
@@ -432,15 +447,19 @@ Run Option panel variables."
 	   (max-size (format "%d%s" (car max-cons) 
 			     (cdr (assoc (cdr max-cons)
 				    memory-unit-abbrevs)))))
-      (if (not (string= start-size "1m"))
-	  (setq options 
-		(nconc options (list (concat "-Xms" start-size)))))
-      (if (not (string= max-size "16m"))
-	  (setq options 
-		(nconc options (list (concat "-Xmx" max-size))))))
+      (append
+       (if (not (string= start-size "1m"))
+	   (list (concat "-Xms" start-size)))
+       (if (not (string= max-size "16m"))
+	  (list (concat "-Xmx" max-size))))))
 
-    ;; Set stack size options.
-    (let* ((c-cons (nth 0 jde-run-option-stack-size))
+(defmethod jde-run-stack-size-args ((this jde-run-vm))
+  "Get stack size arguments."
+    (let* ((memory-unit-abbrevs
+	    (list (cons "bytes" "")
+	       (cons "kilobytes" "k")
+	       (cons "megabytes" "m")))
+	   (c-cons (nth 0 jde-run-option-stack-size))
 	   (c-size (format "%d%s" (car c-cons) 
 			       (cdr (assoc (cdr c-cons)
 				      memory-unit-abbrevs))))
@@ -448,34 +467,35 @@ Run Option panel variables."
 	   (java-size (format "%d%s" (car java-cons) 
 			     (cdr (assoc (cdr java-cons)
 				    memory-unit-abbrevs)))))
-      (if (not (string= c-size "128k"))
-	  (setq options 
-		(nconc options (list (concat "-Xss" c-size)))))
-      (if (not (string= java-size "400k"))
-	  (setq options 
-		(nconc options (list (concat "-Xoss" java-size))))))
+      (append
+       (if (not (string= c-size "128k"))
+	   (list (concat "-Xss" c-size)))
+       (if (not (string= java-size "400k"))
+           (list (concat "-Xoss" java-size))))))
 
-    ;; Set garbage collection options.
+(defmethod jde-run-gc-args ((this jde-run-vm))
+  "Get garbage collection arguments."
     (let ((no-gc-asynch (not 
 			 (nth 0 jde-run-option-garbage-collection)))
 	  (no-gc-classes (not 
 			  (nth 1 jde-run-option-garbage-collection))))
-      (if no-gc-asynch
-	  (setq options (nconc options '("-Xnoasyncgc"))))
-      (if no-gc-classes
-	  (setq options (nconc options '("-Xnoclassgc")))))
+      (append
+       (if no-gc-asynch
+	  '("-Xnoasyncgc"))
+       (if no-gc-classes
+	   '("-Xnoclassgc")))))
 
-    ;; Set Java profile option.
+(defmethod jde-run-java-profile-arg ((this jde-run-vm))
+   "Get Java profile option."
     (let ((profilep (car jde-run-option-java-profile))
 	  (file (cdr jde-run-option-java-profile)))
       (if profilep
 	  (if (string= file "./java.prof")
-	      (setq options (nconc options '("-Xprof")))
-	    (setq options 
-		  (nconc options 
-			 (list (concat "-Xprof:" file)))))))
+	      '("-Xprof")
+	    (list (concat "-Xprof:" file))))))
 
-    ;; Set heap profile option.
+(defmethod jde-run-heap-profile-arg ((this jde-run-vm))
+  "Get heap profile argument."
     (let* ((profilep (car jde-run-option-heap-profile))
 	   (prof-options (cdr jde-run-option-heap-profile))
 	   (file (nth 0 prof-options))
@@ -488,38 +508,113 @@ Run Option panel variables."
 		   (equal depth 5)
 		   (equal top 20)
 		   (string= sort "a"))
-	      (setq options (nconc options '("-Xhprof")))
-	    (setq options
-		  (nconc options
-			 (list
-			  (format 
-			   "-Xhprof:file=%s,depth=%d,top=%d,sort=%s"
-			   file depth top sort)))))))
+	      '("-Xhprof")
+	    (list
+	     (format 
+	      "-Xhprof:file=%s,depth=%d,top=%d,sort=%s"
+	      file depth top sort))))))
 
-    ;; Set verify options.
+(defmethod jde-run-verify-args ((this jde-run-vm))
+  "Get verify arguments."
     (let ((verify-all (nth 0 jde-run-option-verify))
 	  (verify-remote (nth 1 jde-run-option-verify)))
-      (if verify-all
-	  (setq options (nconc options '("-Xverify"))))
-;      (if verify-remote
-;	  (setq options (concat options "-Xverifyremote")))
-      (if (and
+      (append
+       (if verify-all
+	  '("-Xverify"))
+;;       (if verify-remote
+;;	   '("-Xverifyremote"))
+       (if (and
 	   (not verify-all)
 	   (not verify-remote))
-	  (setq options (nconc options '("-Xnoverify")))))
+	   '("-Xnoverify")))))
 
-    ;; Set command line args.
-    (if jde-run-option-vm-args
-	(let ((len (length jde-run-option-vm-args))
-	      (n 0))
-	  (while (< n len)
-	    (setq options (nconc options
-				 (jde-run-parse-args
-				  (nth n jde-run-option-vm-args))))
-	    (setq n (1+ n)))))
-	      
-    options))
+(defmethod jde-run-debugger-connect-args ((this jde-run-vm))
+  "Get arguments required to allow process to connect
+to a debugger."
+  (if (car jde-run-option-connect-to-debugger)
+      (let ((mode (nth 1 jde-run-option-connect-to-debugger))
+	     (address (nth 2 jde-run-option-connect-to-debugger))
+	     (ms-windows (eq system-type 'windows-nt)))
+	(list "-Xdebug"
+	      (format 
+	       "-Xrunjdwp:transport=%s,address=%s,server=%s,suspend=n"
+	       (if ms-windows "dt_shmem" "dt_socket")
+	       (if address address (if ms-windows "javadebug" "4444"))
+	       (if (string= mode "Attach") "y" "n"))))))
 
+(defmethod jde-run-vm-args ((this jde-run-vm))
+  "Get command line args."
+  jde-run-option-vm-args)
+
+(defmethod jde-run-get-vm-args ((this jde-run-vm))
+  (append
+   ;; Classic mode argument must come first.
+   (jde-run-classic-mode-arg this)
+   (jde-run-classpath-arg this)
+   (jde-run-verbose-arg this)
+   (jde-run-property-args this)
+   (jde-run-heap-size-args this)
+   (jde-run-stack-size-args this)
+   (jde-run-gc-args this)
+   (jde-run-java-profile-arg this)
+   (jde-run-heap-profile-arg this)
+   (jde-run-verify-args this)
+   (jde-run-debugger-connect-args this)
+   (jde-run-vm-args this)))
+
+(defmethod jde-run-vm-launch ((this jde-run-vm))
+  (let ((run-buf-name (concat "*" (oref this :main-class) "*"))
+	(source-directory default-directory)
+	(working-directory (if (string= jde-run-working-directory "")
+			       default-directory
+			     (jde-normalize-path 'jde-run-working-directory))))
+    (if (not (comint-check-proc run-buf-name))
+	(let* ((run-buffer (get-buffer-create run-buf-name))
+	       (win32-p (eq system-type 'windows-nt))
+	       (prog (oref this :path))
+	       (prog-args (append
+			   (jde-run-get-vm-args this)
+			   (if jde-run-read-vm-args
+			       (jde-run-parse-args
+				(read-from-minibuffer
+				 "Vm args: "
+				 (car jde-run-interactive-vm-arg-history)
+				 nil nil
+				 'jde-run-interactive-vm-arg-history)))
+			   (list (oref this :main-class))
+			   jde-run-option-application-args
+			   (if jde-run-read-app-args
+			       (jde-run-parse-args
+				(read-from-minibuffer
+				 "Application args: "
+				 (car jde-run-interactive-app-arg-history)
+				 nil nil
+				 'jde-run-interactive-app-arg-history)))
+			   ))
+	       (command-string (concat prog " " 
+				       (jde-run-make-arg-string
+					prog-args)
+				       "\n\n")))
+	  (save-excursion
+	    (set-buffer run-buffer)
+	    (erase-buffer)
+	    (cd working-directory)
+	    (insert (concat "cd " working-directory "\n"))
+	    (insert command-string)
+	    (jde-run-mode))
+	  (let ((win32-start-process-show-window t)
+		(w32-start-process-show-window t)
+		(w32-quote-process-args ?\")
+		(win32-quote-process-args ?\") ;; XEmacs
+		(windowed-process-io t))
+	    (comint-exec run-buffer (oref this :main-class) prog nil prog-args))
+	  (pop-to-buffer run-buffer)
+          (save-excursion
+            (goto-char (point-min))
+          (jde-run-etrace-update-current-marker))
+	  (cd source-directory))
+      (message "An instance of %s is running." (oref this :main-class))
+      (pop-to-buffer run-buf-name))))
 
 ;;;###autoload
 (defun jde-run (main-class)
@@ -529,21 +624,23 @@ not the null string. Otherwise run the class specified by
 the current buffer. Specifying a prefix argument, e.g.,
 C-u C-c C-v C-r, causes this command to prompt you to enter
 the name of the application's main class This command 
-creates a comint buffer to allow you to interacti with the program."
+creates a comint buffer to allow you to interact with the program."
   (interactive "P")
   (if (equal major-mode 'jde-mode)
-      (cond
-       (main-class
-	(jde-run-internal
-	 (read-from-minibuffer
-	  "Main class: "
-	  (concat (jde-db-get-package)
+      (if (string= jde-run-executable "")
+	  (let ((vm (jde-run-get-vm)))
+	    (oset
+	     vm
+	     :main-class
+	     (if main-class
+		 (read-from-minibuffer
+		  "Main class: "
+		  (concat (jde-db-get-package)
 		      (file-name-sans-extension 
-		       (file-name-nondirectory (buffer-file-name)))))))
-      ((string= jde-run-executable "")
-       (jde-run-main-class))
-      (t 
-       (jde-run-executable)))
+		       (file-name-nondirectory (buffer-file-name)))))
+	       (jde-run-get-main-class)))
+	    (jde-run-vm-launch vm))
+	(jde-run-executable))
     (error "The jde-run command works only in a Java source buffer.")))
 
 (defun jde-run-get-main-class () 
@@ -559,14 +656,65 @@ source buffer belongs."
 		       (file-name-nondirectory (buffer-file-name))))))
     main-class))
 
+(defvar jde-run-virtual-machines
+  (list
+   (jde-run-vm "JDK vm"))
+  "*List of supported virtual machines.")
 
+(defun jde-run-get-vm ()
+  "Gets the vm for the current JDK."
+  (let* ((jdk-version (jde-java-version))
+	 (vm
+	  (find-if
+	   (lambda (vm-x)
+	     (string-match
+	      (oref vm-x :version)
+	      (if jdk-version jdk-version "")))
+	   jde-run-virtual-machines)))
+    (if (not vm)
+	(setq vm (car jde-run-virtual-machines)))
+    (oset vm 
+	  :path 
+	  (let ((vm-path 
+		 (substitute-in-file-name jde-vm-path)))
+	    (if (string= vm-path "")
+		(let ((jdk-dir (jde-get-jdk-dir)))
+		  (if jdk-dir
+		      (progn
+			 (setq vm-path 
+			       (expand-file-name 
+				(if (eq system-type 'windows-nt) 
+				    "bin/javaw.exe" 
+				  "bin/java") 
+				jdk-dir))
+			 (if (file-exists-p vm-path)
+			     vm-path
+			   (error (format "Invalid vm path %s" 
+					  vm-path))))
+		       (if (executable-find "java")
+			   (if (eq system-type 'windows-nt) "javaw" "java")
+			 (error "Cannot find a Java vm."))))
+		  (if (file-exists-p
+		       (if (and
+			    (eq system-type 'windows-nt)
+			    (not (string-match "[.]exe$" vm-path)))
+			   (concat vm-path ".exe")
+			 vm-path))
+		      vm-path
+		    (if (executable-find vm-path)
+			vm-path
+		      (error "Invalid vm path: %s"
+			     vm-path))))))
+    vm))
 
 (defun jde-run-main-class()
   "Runs the Java program named by `jde-run-application-class' in
 a buffer, piping output from the program to the buffer and 
 input from the buffer to the program."
   (interactive)
-    (jde-run-internal (jde-run-get-main-class)))
+  (let ((vm (jde-run-get-vm)))
+    (oset vm :main-class (jde-run-get-main-class))
+    (jde-run-vm-launch vm)))
 
 
 (defmacro save-w32-show-window (&rest body)
@@ -588,78 +736,31 @@ buffer belongs is running."
   (let ((run-buf-name (concat "*" (jde-run-get-main-class) "*")))
     (comint-check-proc run-buf-name)))
 
-(defun jde-run-internal(app-class)
-  (let ((run-buf-name (concat "*" app-class "*"))
-	(source-directory default-directory)
-	(working-directory (if (string= jde-run-working-directory "")
-			       default-directory
-			     (jde-normalize-path 'jde-run-working-directory))))
-    (if (not (comint-check-proc run-buf-name))
-	(let* ((run-buffer (get-buffer-create run-buf-name))
-	       (win32-p (eq system-type 'windows-nt))
-	       (prog (if (and
-			  win32-p
-			  (string= jde-run-java-vm "java"))
-			 jde-run-java-vm-w
-		       jde-run-java-vm))
-	       (prog-args (append
-			   (jde-run-get-vm-args)
-			   (if jde-run-read-vm-args
-			       (jde-run-parse-args
-				(read-from-minibuffer
-				 "Vm args: "
-				 (car jde-run-interactive-vm-arg-history)
-				 nil nil
-				 'jde-run-interactive-vm-arg-history)))
-			   (list app-class)
-			   jde-run-option-application-args
-			   (if jde-run-read-app-args
-			       (jde-run-parse-args
-				(read-from-minibuffer
-				 "Application args: "
-				 (car jde-run-interactive-app-arg-history)
-				 nil nil
-				 'jde-run-interactive-app-arg-history)))
-			   ))
-	       (command-string (concat prog " " 
-				       (jde-run-make-arg-string
-					prog-args)
-				       "\n\n")))
-	  (save-excursion
-	    (set-buffer run-buffer)
-	    (erase-buffer)
-	    (cd working-directory)
-	    (insert (concat "cd " working-directory "\n"))
-	    (insert command-string)
-	    (jde-run-mode))
-; 	  (save-w32-show-window
-; 	   (comint-exec run-buffer app-class prog nil prog-args))
-	  (let ((win32-start-process-show-window t)
-		(w32-start-process-show-window t)
-		(windowed-process-io t))
-	    (comint-exec run-buffer app-class prog nil prog-args))
-	  (pop-to-buffer run-buffer)
-	  (cd source-directory))
-      (message "An instance of %s is running." app-class)
-      (pop-to-buffer run-buf-name))))
 
 (defun jde-run-executable()
-  (let* ((prog-name (file-name-sans-extension
-		     (file-name-nondirectory jde-run-executable)))
+  (let* ((prog-path
+	  (jde-normalize-path 
+	   jde-run-executable
+	   'jde-run-executable))
+	 (prog-name (file-name-sans-extension
+		     (file-name-nondirectory 
+		      prog-path)))
 	 (run-buf-name (concat "*" prog-name "*"))
 	 (source-directory default-directory)
 	 (working-directory (if (string= jde-run-working-directory "")
 				default-directory
 			      (jde-normalize-path 'jde-run-working-directory))))
     (if (not (comint-check-proc run-buf-name))
-	(let* ((run-buffer (get-buffer-create run-buf-name))
+	(let* ((w32-quote-process-args ?\")
+	       (win32-quote-process-args ?\") ;; XEmacs
+	       (run-buffer (get-buffer-create run-buf-name))
 	       (prog-args (append
 			   jde-run-executable-args
 			   (if jde-run-read-app-args
 			       (jde-run-parse-args
 				(read-from-minibuffer
 				 "Application args: "
-				 jde-run-interactive-app-args
+				 nil
 				 nil nil
 				 '(jde-run-interactive-app-arg-history . 1))))
 			   ))
@@ -675,13 +776,15 @@ buffer belongs is running."
 	    (insert command-string)
 	    (jde-run-mode))
 	  (save-w32-show-window
-	   (comint-exec run-buffer prog-name jde-run-executable nil prog-args))
+	   (comint-exec run-buffer prog-name prog-path nil prog-args))
 	  (pop-to-buffer run-buffer)
+          (save-excursion
+            (goto-char (point-min))
+            (jde-run-etrace-update-current-marker))
 	  (cd source-directory))
       (message "An instance of %s is running." prog-name)
       (pop-to-buffer run-buf-name))))
 
-
 (defun jde-run-mode-internal()
   "Mode for running Java programs."
   (define-key (current-local-map) "\C-c\C-v\C-[" 'jde-run-etrace-prev)
@@ -739,7 +842,9 @@ under Windows."
 ;; the appropriate environment.
 
 (defun jde-run-applet-exec-1 (name buffer command switches)
-  (let ((process-environment
+  (let ((w32-quote-process-args ?\")
+	(win32-quote-process-args ?\") ;; XEmacs
+	(process-environment
 	 (nconc
 	  ;; If using termcap, we specify `emacs' as the terminal type
 	  ;; because that lets us specify a width.
@@ -850,7 +955,10 @@ file."
 	  (if (or
 	       (string= jde-run-applet-viewer "")
 	       (string-match "browse-url" jde-run-applet-viewer))
-	      (browse-url applet-doc browse-url-new-window-p)
+	      (browse-url applet-doc 
+			  (if (boundp 'browse-url-new-window-flag)
+			      'browse-url-new-window-flag
+			    browse-url-new-window-p))
 	    (jde-run-applet-internal (concat default-directory applet-doc))))
       (signal 'error "Could not find html document to display applet."))))
 
@@ -873,39 +981,62 @@ file."
   "The location of the last stack shown.
 A cons of two markers, location of the error and the location in the code.")
 
+(defun jde-run-etrace-update-current-marker () 
+  "Updates the `car' of `jde-run-etrace-current-marker' to be at the current point"
+  (set-marker (car jde-run-etrace-current-marker) (point) (current-buffer)))
 
 (defun jde-run-etrace-current-marker (&optional next)
-  "Update the `cdr' of `jde-run-etrace-current-marker' from its `car'.
+  "Update the `cdr' and the `car' of `jde-run-etrace-current-marker' from its `car'.
 Here goes all the error message parsing."
   (let ((here (car jde-run-etrace-current-marker))
-        (there (cdr jde-run-etrace-current-marker)))
+        (there (cdr jde-run-etrace-current-marker))
+        (n (or next 0))
+        (re (concat " \\([a-zA-Z0-9_.]+\\.\\)?" ; package
+                     "\\([a-zA-Z0-9_$]+\\)" ; class
+                     "\\.<?[a-zA-Z0-9_]+>?" ; method
+                     "(\\([a-zA-Z0-9_]+\\)\\.java:" ; java file = public class
+                     "\\([0-9]+\\))"))); line number
     (save-excursion
       (set-buffer (marker-buffer here))
       (goto-char here)
-      (forward-line (or next 0))
-      (re-search-forward
-       (eval-when-compile
-        (concat "\\([a-zA-Z0-9_.]+\\.\\)?" ; package
-                "\\([a-zA-Z0-9_]+\\)" ; class
-                "\\.<?[a-zA-Z0-9_]+>?" ; method
-                "(\\([a-zA-Z0-9_]+\\)\\.java:" ; java file = public class
-                "\\([0-9]+\\))"))
-       nil t) ; line number
-      (message "1: [%s]; 2: [%s]; 3: [%s]; 4: [%s]" (match-string 1)
-               (match-string 2) (match-string 3) (match-string 4))
+      ;; In order to display the current match at the top of 
+      ;; the error screen the previous match was set to be 
+      ;; at the beggining of the line. If we do not place the point
+      ;; at the end of the line again, you will match the same error.
+      ;; If n is 0 ensure that point is at the beggining
+      (if (> n 0)
+          (end-of-line)
+        (beginning-of-line))
+      
+      (if (>= n 0)
+          (if (re-search-forward re nil t)
+              (progn
+                ;;setting the line at the beggining
+                ;;so that it is the first line of the errors
+                (beginning-of-line) 
+                (jde-run-etrace-update-current-marker))
+            (error "End of stack trace"))
+        (if (re-search-backward re nil t)
+            (progn
+              (jde-run-etrace-update-current-marker))
+          (error "Start of stack trace")))
+;;       (message "1: [%s]; 2: [%s]; 3: [%s]; 4: [%s]" (match-string 1)
+;;                (match-string 2) (match-string 3) (match-string 4))
       (let* ((package (or (match-string 1) ""))
             (class (match-string 2))
             (file-name (match-string 3))
             (line (car (read-from-string (match-string 4))))
             (file (jde-find-class-source-file (concat package file-name)))
-	    (buf (if file (find-file-noselect file)
-                    (error "jde-class-source cannot find source for %s%s (%s)"
-                           package class file-name))))
-	(set-buffer buf) (goto-line line)
-	(set-marker there (point) buf)))
+	    buf) 
+        (condition-case err
+            (progn 
+              (setq buf (if file (find-file-noselect file)))
+              (set-buffer buf)
+              (goto-line line)
+              (set-marker there (point) buf))
+          (error err))))
     jde-run-etrace-current-marker))
 
-
 (defun jde-run-etrace-goto (&optional next)
   "Display the current stack using `compilation-goto-locus'."
   (compilation-goto-locus (jde-run-etrace-current-marker next)))
@@ -915,9 +1046,10 @@ Here goes all the error message parsing."
   "Jump to the stack position at the mouse click.
 Click anywhere on the line with the stack reference."
   (interactive "e")
-  (set-marker (car jde-run-etrace-current-marker)
-              (posn-point (event-start event))
-              (window-buffer (posn-window (event-start event))))
+  (let ((pos (event-start event)))
+    (set-marker (car jde-run-etrace-current-marker)
+                (posn-point pos)
+                (window-buffer (posn-window pos))))
   (jde-run-etrace-goto))
 
 
@@ -930,13 +1062,13 @@ The point should be anywhere on the line with the stack reference."
 
 
 (defun jde-run-etrace-next ()
-  "Jump to the next stack postion (next line)."
+  "Jump to the next stack position (next line)."
   (interactive)
   (jde-run-etrace-goto 1))
 
 
 (defun jde-run-etrace-prev ()
-  "Jump to the previous stack postion (previous line)."
+  "Jump to the previous stack position (previous line)."
   (interactive)
   (jde-run-etrace-goto -1))
 
@@ -952,34 +1084,123 @@ The point should be anywhere on the line with the stack reference."
   ;; stack entry file -> font-lock-variable-name-face
   ;; stack entry line number -> font-lock-type-face
 
-  (if jde-xemacsp
+  (if (featurep 'xemacs)
       nil
     (progn
       (font-lock-add-keywords 
        nil 
-       '(("\\(^[a-z.]+[a-zA-Z0-9]+Exception\\)\\(: \\)?\\(.*\\)?"
+       '(("\\(^[_a-z.]+[_a-zA-Z0-9]+Exception\\)\\(: \\)?\\(.*\\)?"
 	  (1 'font-lock-keyword-face append)
 	  (3 'font-lock-string-face append))))
   
       (font-lock-add-keywords 
        nil 
-       '(("\\(at [a-z.]+[a-zA-Z0-9]+\\.[a-zA-Z<>]*\\)(\\([a-zA-Z0-9]+.java\\):\\([0-9]+\\))$"
+       '(("\\(at [_a-z.]+[_a-zA-Z0-9]+\\.[_a-zA-Z<>]*\\)(\\([_a-zA-Z0-9]+.java\\):\\([0-9]+\\))$"
 	  (1 'font-lock-constant-face append)
 	  (2 'font-lock-variable-name-face append)
 	  (3 'font-lock-type-face append))))
       (font-lock-fontify-buffer))))
 
-;;(define-key jde-mode-map "\C-c\C-v\C-[" 'jde-run-etrace-prev)
-;;(define-key jde-mode-map "\C-c\C-v\C-]" 'jde-run-etrace-next)
-
-
 (provide 'jde-run)
 
 
 ;; Change History
 ;; $Log: jde-run.el,v $
-;; Revision 1.4  2001/08/16 03:29:21  andyp
-;; Update to JDE 2.2.7.1
+;; Revision 1.6  2002/09/14 03:21:05  andyp
+;; sync to jde 2.2.9beta12
+;;
+;; Revision 1.73  2002/09/10 13:51:47  jslopez
+;; Fixes regression bug in jde-run-etrace-set-font-lock.
+;;
+;; Revision 1.72  2002/09/08 14:33:58  jslopez
+;; Fixes etrace highlighting of classes that contains underscores.
+;; Thanks to Laurent Martelli <laurent@bearteam.org>
+;;
+;; Revision 1.71  2002/08/20 02:34:50  jslopez
+;; Fixes bug in etrace command that did not recognize inner classes as valid
+;; stack positions.
+;;
+;; Revision 1.70  2002/08/07 06:36:17  paulk
+;; Removed code intended to eliminate spurious warnings when byte-compiling the JDEE. The
+;; removed code now resides in a separate package, jde-compat.el. Thanks to Andy Piper
+;; for suggesting this restructuring. Also fixed a number of compiler warnings caused
+;; by genuine bugs.
+;;
+;; Revision 1.69  2002/06/12 07:04:23  paulk
+;; XEmacs compatibility fix: set win32-quote-process-args wherever
+;; the JDEE sets w32-quote-process-args. This allows use of spaces in
+;; paths passed as arguments to processes (e.g., javac)  started by
+;; the JDEE.
+;;
+;; Revision 1.68  2002/06/11 06:26:14  paulk
+;; Provides support for paths containing spaces as vm arguments via the following change:
+;; locally set the w32-quote-process-args variable to a quotation mark when launching
+;; the vm process.
+;;
+;; Revision 1.67  2002/03/24 05:30:10  paulk
+;; Fixed regression error in jde-run command that prevented use of the prefix
+;; argument to cause the command to prompt the user for the main class.
+;;
+;; Revision 1.66  2002/03/08 13:14:21  paulk
+;; Fixed bug in jde-run-debugger-connect-args that generates an incorrect
+;; -Xrunjdwp argument on Unix. Thanks to Jens Lautenbacher.
+;;
+;; Revision 1.65  2002/03/05 10:57:22  paulk
+;; Fixed bug in jde-run-debugger-connect-args.
+;;
+;; Revision 1.64  2002/03/04 06:46:51  paulk
+;; Adds jde-run-option-connect-to-debugger variable
+;;
+;; Revision 1.63  2002/02/08 11:52:17  paulk
+;; The jde-run-executable command now normalizes the path to the executable.
+;;
+;; Revision 1.62  2001/12/08 04:11:26  paulk
+;; Updated doc string for jde-run-executable.
+;;
+;; Revision 1.61  2001/12/05 19:16:53  jslopez
+;; Now jde-run-etrace-prev and jde-run-etrace-next
+;; allow you to move through the whole stack trace.
+;; It behaves as compilation-next-error.
+;; Better error handling.
+;;
+;; Revision 1.60  2001/12/05 02:45:34  jslopez
+;; Substitute jde-xemacsp for (featurep 'xemacs).
+;;
+;; Revision 1.59  2001/12/05 02:41:26  jslopez
+;; Fixes typo in the documentation
+;; for jde-run-etrace-prev and
+;; jde-run-etrace-next.
+;;
+;; Revision 1.58  2001/11/30 02:21:41  jslopez
+;; Removes obsolete methods setting jde-run-java-vm
+;; and jde-run-java-vm-w.
+;; Substitutes remaining references to jde-run-java-vm
+;; and jde-run-java-vm-w for jde-run-get-vm.
+;;
+;; Revision 1.57  2001/11/13 07:35:22  paulk
+;; jde-get-java-vm returns the default vm when the JDE cannot determine the
+;; version of the vm to be used.
+;;
+;; Revision 1.56  2001/11/05 05:10:56  paulk
+;; Provided a better fix for classic mode argument.
+;;
+;; Revision 1.55  2001/11/04 14:31:53  paulk
+;; Fixed regression in support for -classic argument.
+;;
+;; Revision 1.54  2001/10/31 08:58:44  paulk
+;; Emacs 21 compatibility fix: added support for browse-url-new-window-flag variable (replaces browse-url-new-window-p in Emacs 21).
+;;
+;; Revision 1.53  2001/10/23 05:31:25  paulk
+;; Replaces the customization variables jde-run-java-vm and
+;; jde-run-java-vm-w with the variable jde-vm-path.  If this variable is
+;; a valid path, the JDE uses this path to find the vm used to run Java
+;; applications.  If this path is the empty string, the JDE runs the vm
+;; included with the version of the JDK specified by jde-jdk, or if
+;; jde-jdk is nil, the first vm on the system command path.
+;;
+;; This change is part of an ongoing enhancement of the JDE to permit
+;; easy switching of JDK's. Now jde-jdk specifies both the compiler and
+;; the vm used by the JDE.
 ;;
 ;; Revision 1.52  2001/04/28 06:22:19  paulk
 ;; Makes jde-run-mode a full-fledged major mode derived from comint-mode.
@@ -1198,4 +1419,4 @@ The point should be anywhere on the line with the stack reference."
 ;; Initial revision
 ;;
 
-;;; jde-run.el ends here
\ No newline at end of file
+;;; jde-run.el ends here
