@@ -1,5 +1,5 @@
 ;;; jde-widgets.el -- Custom-style widgets used by the JDE
-;; $Revision: 1.4 $ $Date: 2001/08/16 03:29:21 $ 
+;; $Revision: 1.6 $ $Date: 2002/09/14 03:21:05 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -39,7 +39,7 @@
 
 (require 'wid-edit)
 (require 'eieio)
-(require 'cl)
+(require 'efc)
 (require 'tree-widget)
 
 
@@ -184,117 +184,6 @@
   (widget-setup))
 
 
-(defclass jde-dialog ()
-  ((title     :initarg :title
-	      :type string
-	      :initform "JDE Dialog"
-	      :documentation
-	      "Title of dialog")
-   (buf       :initarg :buf
-	      :type buffer
-	      :documentation
-	      "Dialog buffer")
-   )
-  "Super class of JDE dialogs."
-  )
-
-(defmethod initialize-instance ((this jde-dialog) &rest fields)
-  "Constructor for trace methods dialog."
-
-  ;; Call parent initializer.
-  (call-next-method)
-
-  (oset this buf (get-buffer-create (oref this title)))
-  (set-buffer (oref this buf))
-
-  (jde-dialog-create this)
-
-  (widget-put
-   (widget-create 
-    'push-button
-    :notify 
-    (lambda (button &rest ignore) (jde-dialog-ok (widget-get button :dialog)))
-    "Ok")
-   :dialog this)
-
-  (widget-insert "  ")
-
-  (widget-put
-   (widget-create 
-    'push-button
-    :notify (lambda (button &rest ignore) (jde-dialog-cancel (widget-get button :dialog)))
-    "Cancel")
-   :dialog this)
-
-  (use-local-map widget-keymap)
-  (widget-setup)
-  )
-
-(defmethod jde-dialog-create ((this jde-dialog)))
-
-(defmethod jde-dialog-ok ((this jde-dialog))
-  "Invoked when the user clicks the dialog's okay button. The
-default method kills the dialog buffer."
-  (kill-buffer (current-buffer)))
-
-(defmethod jde-dialog-cancel ((this jde-dialog))
-  "Invoked when the user clicks the dialog's Cancel button. The
-default method kills the dialog buffer."
-  (kill-buffer (current-buffer)))
-
-(defmethod jde-dialog-show ((this jde-dialog))
-   (pop-to-buffer (oref this buf)))
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                                                                            ;; 
-;; Option Dialog                                                              ;;
-;;                                                                            ;;
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defclass jde-option-dialog (jde-dialog)
-  ((options       :initarg :options
-		  :documentation
-		  "Options from from which to choose.")		  
-   (radio-buttons :initarg :interface-buttons
-		  :documentation
-		  "Buttons for selecting options.")
-   (text          :initarg :text
-		  :type string
-		  :initform "Select option."
-		  :documentation
-		  "Text to be inserted at top of dialog.")
-   (ok-action     :initarg :ok-action
-		  :type function
-		  :documentation
-		  "Function to invoke when the user selects the ok button."))
-   "This dialog allows a user to choose one of a set of OPTIONS by clicking
-a radio button next to the option. The dialog invokes the function OK-ACTION
-when the user selects the OK button on the dialog. OK-ACTION must be a function
-that takes the selected option as its only argument.")
-
-(defmethod initialize-instance ((this jde-option-dialog) &rest fields)
-  "Dialog constructor."
-  (call-next-method))
-
-(defmethod jde-dialog-create ((this jde-option-dialog))
-  (widget-insert (oref this text))
-  (widget-insert "\n\n")
-  (oset this radio-buttons
- 	(widget-create
- 	 (list
-	  'radio-button-choice
-	  :value (car (oref this options))
-	  :args (mapcar 
-		 (lambda (x) 
-		   (list 'item x)) 
-		 (oref this options)))))
-  (widget-insert "\n"))
-
-(defmethod jde-dialog-ok ((this jde-option-dialog))
-  (funcall (oref this ok-action) 
-	   (widget-value (oref this radio-buttons))))
-
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;; 
@@ -393,19 +282,38 @@ tree."
 		       :process process
 		       :object-id (oref var-value id)))
 		 (str-val (jde-dbs-cmd-exec cmd)))
-	    (list 'tree-widget :tag var-tag :value t
-		  (list 'tree-widget :tag str-val)))
-	(list 'jde-widget-java-obj :tag var-tag
-	      :process process :object-id (oref var-value :id))))
+	    (list 'tree-widget
+                  :tag var-tag
+                  :node-name var-tag
+                  :open (jde-dbo-locals-open-p var-tag)
+                  :value t
+                  (list 'tree-widget :tag str-val)))
+	(list 'jde-widget-java-obj
+              :tag var-tag
+              :node-name var-tag
+              :open (jde-dbo-locals-open-p var-tag)
+	      :process process
+              :object-id (oref var-value :id))))
      ((typep var-value 'jde-dbs-java-array)
-      (list 'jde-widget-java-array :tag var-tag
+      (list 'jde-widget-java-array
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
 	    :process process :object var-value))
      ((typep var-value 'jde-dbs-java-primitive)
-      (list 'tree-widget :tag var-tag :value t
+      (list 'tree-widget
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
+            :value t
 	    (list 'tree-widget 
 		  :tag (format "%s" (oref var-value value)))))
      ((typep var-value 'jde-dbs-java-null)
-      (list 'tree-widget :tag var-tag :value t
+      (list 'tree-widget
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
+            :value t
 	    (list 'tree-widget :tag "null")))
      (t
       (error "Unidentified type of local variable: %s" var-tag)))))
@@ -457,14 +365,24 @@ the object exists, and  ID is the debugger id for the object."
 		       :process process
 		       :object-id (oref element id)))
 		 (str-val (jde-dbs-cmd-exec cmd)))
-	    (list 'tree-widget :tag (format "[%d] %s" index str-val)))
+	    (list 'tree-widget
+                  :tag (format "[%d] %s" index str-val)
+                  :node-name (format "[%d] %s" index str-val)
+                  :open (jde-dbo-locals-open-p (format "[%d] %s" index str-val))
+                  ))
 	(list 'jde-widget-java-obj 
 	      :tag (format "[%d] %s" index (oref element jtype))
-	      :process process :object-id (oref element id))))
+              :node-name (format "[%d] %s" index (oref element jtype))
+              :open (jde-dbo-locals-open-p (format "[%d] %s" index (oref element jtype)))
+	      :process process
+              :object-id (oref element id))))
      ((typep element 'jde-dbs-java-array)
       (list 'jde-widget-java-array 
 	    :tag (format "[%d] %s" index (oref element jtype))
-	    :process process :object element))
+            :node-name (format "[%d] %s" index (oref element jtype))
+            :open (jde-dbo-locals-open-p (format "[%d] %s" index (oref element jtype)))
+	    :process process
+            :object element))
      ((typep element 'jde-dbs-java-primitive)
       (list 'tree-widget :tag (format "[%d] %s"  index (oref element value))))
      ((typep element 'jde-dbs-java-null)
@@ -664,8 +582,57 @@ expand button causes the widget to display the values of the array."
 (provide 'jde-widgets)
 
 ;; $Log: jde-widgets.el,v $
-;; Revision 1.4  2001/08/16 03:29:21  andyp
-;; Update to JDE 2.2.7.1
+;; Revision 1.6  2002/09/14 03:21:05  andyp
+;; sync to jde 2.2.9beta12
+;;
+;; Revision 1.24  2001/12/04 05:27:26  paulk
+;; Moved dialog classes to the efc (Emacs Foundation Classes) package.
+;;
+;; Revision 1.23  2001/11/29 11:41:18  paulk
+;; Option dialog now positions cursor over OK button.
+;;
+;; Revision 1.22  2001/11/28 16:50:28  jslopez
+;; Fixes bug, that would not keep the state
+;; of nodes created by jde-widget-java-array-element-to-tree.
+;;
+;; Revision 1.21  2001/11/28 08:38:02  paulk
+;; Added note about fix for Emacs 21.1 bug.
+;;
+;; Revision 1.20  2001/11/27 21:04:05  jslopez
+;; Modifies jde-widget-java-var-to-tree to keep nodes the states of nodes.(open/close)
+;;
+;; Revision 1.19  2001/11/24 15:23:11  paulk
+;; The jde-option-dialog versions of jde-dialog-ok and jde-dialog-cancel
+;; now delete the other window ony if the current version of Emacs is not
+;; XEmacs and is less than major version 21.
+;;
+;; Revision 1.18  2001/11/24 07:31:56  paulk
+;; The jde-option-dialog version of jde-dialog-ok now deletes the other windows,
+;; if the current version of Emacs is not 21.1.
+;;
+;; Revision 1.17  2001/11/24 07:23:54  paulk
+;; Changed the jde-dialog-ok method of jde-option-dialog to not delete
+;; other windows. This is necessary to avoid a bug in the Emacs 21.1.1
+;; version of exit-recursive-edit. This line should be restored when
+;; the bug is fixed.
+;;
+;; Revision 1.16  2001/10/26 06:36:07  paulk
+;; Changed jde-option-dialog to use recursive-edit mode to emulate a
+;; modal dialog. Now the show dialog method enters recursive edit mode
+;; thereby suspending the currently executing command until the user
+;; selects the OK or Cancel button on the dialog. Selecting the OK button
+;; on the dialog causes the dialog to update its :selection field to the
+;; option chosen by the user. Selecting the Cancel button sets the
+;; :selection field to nil.
+;;
+;; Revision 1.15  2001/08/09 03:37:11  paulk
+;; Fixed error caused by missing :documentatation keyword in jde-option-dialog. Thanks to David Ponce.
+;;
+;; Revision 1.14  2001/08/07 05:31:26  paulk
+;; Added ok-action-args field to jde-option-dialog.
+;;
+;; Revision 1.13  2001/07/31 05:11:52  paulk
+;; ReleaseNotes.txt
 ;;
 ;; Revision 1.12  2001/05/21 06:44:29  paulk
 ;; Added jde-option-dialog class.
@@ -706,4 +673,3 @@ expand button causes the widget to display the values of the array."
 ;;
 
 ;; End of jde-widgets.el
-    
\ No newline at end of file
