@@ -7,7 +7,7 @@
 ;; Created: 16 Feb 2001
 ;; Version: 1.0.5
 ;; Keywords: extensions
-;; VC: $Id: tree-widget.el,v 1.1 2001/08/16 03:29:22 andyp Exp $
+;; VC: $Id: tree-widget.el,v 1.3 2002/09/14 03:21:09 andyp Exp $
 
 ;; This file is not part of Emacs
 
@@ -115,8 +115,26 @@
 ;;; History:
 ;; 
 ;; $Log: tree-widget.el,v $
-;; Revision 1.1  2001/08/16 03:29:22  andyp
-;; Update to JDE 2.2.7.1
+;; Revision 1.3  2002/09/14 03:21:09  andyp
+;; sync to jde 2.2.9beta12
+;;
+;; Revision 1.4  2001/11/27 22:13:47  jslopez
+;; Adding David's change log entry.
+;;
+;; Revision 1.3  2001/11/27 20:56:25  jslopez
+;; Updates to the latest version. David added a hook
+;; to be able to persist the JDEBug local variables tree.
+;; (tree-widget-after-toggle-functions): New variable.  Hooks run
+;; after toggling a `tree-widget' folding.
+;; (tree-widget-toggle-folding): Run above hooks.  Updated doc
+;; string.
+;; Added pagination.  Minor comment changes.
+;;
+;; Revision 1.2  2001/10/26 11:20:38  jslopez
+;; Removing control characters.
+;;
+;; Revision 1.1  2001/10/26 06:45:57  paulk
+;; Initial revision.
 ;;
 ;; Revision 1.5  2001/05/11 23:11:18  ponce
 ;; Updated version to 1.0.5.
@@ -254,9 +272,16 @@ in NODE if non-nil else in WIDGET :node property value."
            ;; Save the node child properties specified in :keep.
            (tree-widget-keep node node-child)))))
 
+(defvar tree-widget-after-toggle-functions nil
+  "Hooks run after toggling a `tree-widget' folding.
+Each function will receive the `tree-widget' as its unique argument.
+This variable should be local to each buffer used to display
+widgets.")
+
 (defun tree-widget-toggle-folding (widget &rest ignore)
-  "Toggle tree WIDGET folding.
-IGNORE other arguments."
+  "Toggle a `tree-widget' folding.
+WIDGET is a `tree-widget-node-handle-widget' and its parent the
+`tree-widget' itself.  IGNORE other arguments."
   (let ((parent (widget-get widget :parent))
         (open   (widget-value widget)))
      (if open
@@ -264,7 +289,8 @@ IGNORE other arguments."
          ;; open can recover them.
          (tree-widget-children-value-save parent))
     (widget-put parent :open (not open))
-    (widget-value-set parent (not open))))
+    (widget-value-set parent (not open))
+    (run-hook-with-args 'tree-widget-after-toggle-functions parent)))
 
 (defvar tree-widget-button-keymap
   (let (parent-keymap mouse-button1 keymap)
@@ -278,7 +304,6 @@ IGNORE other arguments."
     keymap)
   "Keymap used inside node handle buttons.")
 
-
 (define-widget 'tree-widget-node-handle 'toggle
   "Tree node handle widget."
   :button-keymap  tree-widget-button-keymap ; XEmacs
@@ -347,6 +372,7 @@ WIDGET is a tree leaf node and ESCAPE a format character."
 	(open (widget-get widget :open))
 	(node (tree-widget-node widget))
 	children buttons prefix)
+    
     (cond
 
      ;; Leaf node.
@@ -559,4 +585,4 @@ Reuse the cached :args property value if exists."
 
 (provide 'tree-widget)
 
-;;; tree-widget.el ends here 
\ No newline at end of file
+;;; tree-widget.el ends here 
